<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Privacy class for requesting user data.
 *
 * @package    assignsubmission_collabora
 * @copyright  2019 Benjamin Ellis, Synergy Learning
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace assignsubmission_collabora\privacy;

defined('MOODLE_INTERNAL') || die;

global $CFG;

require_once($CFG->dirroot . '/mod/assign/locallib.php');

use core_privacy\local\metadata\collection;
use core_privacy\local\request\contextlist;
use core_privacy\local\request\writer;
use core_privacy\local\metadata\provider as metadata_provider;
use mod_assign\privacy\assignsubmission_provider;
use mod_assign\privacy\assignsubmission_user_provider;
use mod_assign\privacy\assign_plugin_request_data;

/**
 * Privacy class for requesting user data.
 *
 * @package    assignsubmission_collabora
 * @copyright  2019 Benjamin Ellis, Synergy Learning
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class provider implements metadata_provider, assignsubmission_provider, assignsubmission_user_provider {
    /**
     * Return meta data about this plugin.
     *
     * @param  collection $collection a list of information to add to
     * @return collection return the collection after adding to it
     */
    public static function get_metadata(collection $collection): collection {
        $collection->link_subsystem('core_files', 'privacy:metadata:filepurpose');

        return $collection;
    }

    /**
     * This is covered by mod_assign provider and the query on assign_submissions.
     *
     * @param int         $userid      the user ID that we are finding contexts for
     * @param contextlist $contextlist a context list to add sql and params to for contexts
     */
    public static function get_context_for_userid_within_submission(int $userid, contextlist $contextlist) {
        // This is already fetched from mod_assign.
    }

    /**
     * This is also covered by the mod_assign provider and it's queries.
     *
     * @param \mod_assign\privacy\useridlist $useridlist an object for obtaining user IDs of students
     */
    public static function get_student_user_ids(\mod_assign\privacy\useridlist $useridlist) {
        // No need.
    }

    /**
     * If you have tables that contain userids and you can generate entries in your tables without creating an
     * entry in the assign_submission table then please fill in this method.
     *
     * @param \core_privacy\local\request\userlist $userlist The stdClass userlist object
     */
    public static function get_userids_from_context(\core_privacy\local\request\userlist $userlist) {
        // Not required.
    }

    /**
     * Export all user data for this plugin.
     *
     * @param assign_plugin_request_data $exportdata data used to determine which context and user to export and other useful
     *                                               information to help with exporting
     */
    public static function export_submission_user_data(assign_plugin_request_data $exportdata) {
        // We currently don't show submissions to teachers when exporting their data.
        $context = $exportdata->get_context();
        if ($exportdata->get_user() != null) {
            return null;
        }
        $user       = new \stdClass();
        $assign     = $exportdata->get_assign();
        $plugin     = $assign->get_plugin_by_type('assignsubmission', 'collabora');
        $submission = $exportdata->get_pluginobject();
        // We can not use group submissions here!
        if (empty($submission->userid) && !empty($submission->groupid)) {
            return;
        }
        $files = $plugin->get_files($submission, $user);
        foreach ($files as $file) {
            $userid = $exportdata->get_pluginobject()->userid;
            writer::with_context($exportdata->get_context())->export_file($exportdata->get_subcontext(), $file);

            // Plagiarism data.
            $coursecontext = $context->get_course_context();
            \core_plagiarism\privacy\provider::export_plagiarism_user_data($userid, $context, $exportdata->get_subcontext(), [
                'cmid'   => $context->instanceid,
                'course' => $coursecontext->instanceid,
                'userid' => $userid,
                'file'   => $file,
            ]);
        }
    }

    /**
     * Any call to this method should delete all user data for the context defined in the deletion_criteria.
     *
     * @param assign_plugin_request_data $requestdata information useful for deleting user data
     */
    public static function delete_submission_for_context(assign_plugin_request_data $requestdata) {
        \core_plagiarism\privacy\provider::delete_plagiarism_for_context($requestdata->get_context());

        $filearea = \assignsubmission_collabora\api\collabora_fs::FILEAREA_SUBMIT;
        $fs       = get_file_storage();
        $fs->delete_area_files($requestdata->get_context()->id, 'assignsubmission_collabora', $filearea);
    }

    /**
     * A call to this method should delete user data (where practical) using the userid and submission.
     *
     * @param assign_plugin_request_data $deletedata details about the user and context to focus the deletion
     */
    public static function delete_submission_for_userid(assign_plugin_request_data $deletedata) {
        global $DB;

        \core_plagiarism\privacy\provider::delete_plagiarism_for_user($deletedata->get_user()->id, $deletedata->get_context());

        $submissionid = $deletedata->get_pluginobject()->id;

        $fs = get_file_storage();
        $fs->delete_area_files(
            $deletedata->get_context()->id,
            'assignsubmission_collabora',
            \assignsubmission_collabora\api\collabora_fs::FILEAREA_SUBMIT,
            $submissionid
        );

        $DB->delete_records(
            'assignsubmission_collabora',
            [
                'assignment' => $deletedata->get_assignid(),
                'submission' => $submissionid,
            ]
        );
    }

    /**
     * Deletes all submissions for the userids provided in a context.
     *
     * @param assign_plugin_request_data $deletedata a class that contains the relevant information required for deletion
     */
    public static function delete_submissions(assign_plugin_request_data $deletedata) {
        global $DB;

        \core_plagiarism\privacy\provider::delete_plagiarism_for_users($deletedata->get_userids(), $deletedata->get_context());

        if (empty($deletedata->get_submissionids())) {
            return;
        }
        $fs                 = get_file_storage();
        list($sql, $params) = $DB->get_in_or_equal($deletedata->get_submissionids(), SQL_PARAMS_NAMED);
        $fs->delete_area_files_select(
            $deletedata->get_context()->id,
            'assignsubmission_collabora',
            \assignsubmission_collabora\api\collabora_fs::FILEAREA_SUBMIT,
            $sql,
            $params
        );

        $params['assignid'] = $deletedata->get_assignid();
        $DB->delete_records_select('assignsubmission_collabora', "assignment = :assignid AND submission $sql", $params);
    }
}
